> {
> -----------------------------------------------------------------------------
> -- |
> -- Module      :  LRSX.Interface.Parser
> -- Copyright   :  (c) D. Sabel, Goethe-University, Frankfurt a.M., Germany
> -- License     :  BSD-style 
> -- 
> -- Maintainer  :  sabel <at> ki.cs.uni-frankfurt.de
> --
> --  
> -- The Parser of LRSX Unification tool, generated by Happymainly
> --
> -----------------------------------------------------------------------------
> -- 
> --
> -- other todos:  - add strictness and arity info automatically 
> --               - nub funinfo
> --               - check that all binding positions are correct
> --               - check that strictness information and arities are consistently used
> --               - check that no binding position is strict
> --               - compute the bindarity
> --
> module LRSX.Interface.Parser 
> where
> import LRSX.Interface.Lexer
> import LRSX.Language.Syntax 
> import Data.List
> import LRSX.Unification.UnificationProblem
> import LRSX.Diagrams.Rules

> }

> %name parseExpr Expr
> %name parse EqConstr
> %name parseMatchProblem EqTwoConstr
> %name parseRules Rules
> %name parseRulesAndCommands RulesAndCommands
> %name parseSRulesAndCommands SRulesAndCommands
> %tokentype { TokenType }

> %token 
> var               { TokenVar  $$ _                   }
> metavar           { TokenMetaVar $$ _                }
> metavarACtxt      { TokenMetaVarCtxt ACtxt $$ _      }
> metavarCCtxt      { TokenMetaVarCtxt Ctxt  $$ _      }
> metavarTCtxt      { TokenMetaVarCtxt TCtxt $$ _      }
> '|'               { TokenSymbol   "|" _              }
> ','               { TokenSymbol   "," _              }
> '\\'              { TokenSymbol   "\\" _             }
> ';'               { TokenSymbol   ";" _              }
> '='               { TokenSymbol   "=" _              }
> '/='              { TokenSymbol   "/=" _             }
> '==>'             { TokenSymbol  "==>" _             }
> '.'               { TokenSymbol  "." _               }
> '('               { TokenSymbol  "("  _              }
> '{'               { TokenSymbol  "{"  _              }
> ')'               { TokenSymbol  ")" _               }
> '}'               { TokenSymbol  "}" _               }
> '['               { TokenSymbol  "["  _              }
> ']'               { TokenSymbol  "]"  _              }
> '*'               { TokenSymbol  "*"  _              }
> '+'               { TokenSymbol  "+"  _              }
> '.l'              { TokenSymbol ".l" _               }
> '.r'              { TokenSymbol ".r" _               }
> '=?='             { TokenSymbol  "=?="  _            }
> 'letrec'          { TokenKeyword "letrec" _          }
> 'TRANSFORMATIONS' { TokenKeyword "TRANSFORMATIONS" _ }
> 'in'              { TokenKeyword "in" _              }
> 'overlap'         { TokenKeyword "overlap" _         }
> 'all'             { TokenKeyword "all" _         }
> 'and'             { TokenKeyword "and" _             }
> 'HOLE'            { TokenKeyword "HOLE" _            }
> 'vlet'            { TokenKeyword "vlet" _            }
> 'vlift'           { TokenKeyword "vlift" _            }
> 'var'             { TokenKeyword "var" _            }
> 'vlam'            { TokenKeyword "vlam" _            }
> 'where'           { TokenKeyword "where" _           }
> 'except'          { TokenKeyword "except" _          }
> 'union'          { TokenKeyword "union" _          }
> 'ignore'          { TokenKeyword "ignore" _          }
> 'ANSWER'          { TokenKeyword "ANSWER" _          }
> 'SR'              { TokenKeyword "SR" _          }
> num               { TokenNumber $$ _}
> %right '.'
> %nonassoc ';'
> %nonassoc 'in' 
> %nonassoc 'letrec'
> %%

> SRulesAndCommands ::  {((([ConstrainedRule],[ConstrainedRule],[ConstrainedRule]),[Command],[Command]),FunInfo)}
> SRulesAndCommands : SRules                               {let (rs,finfo) = $1 in ((rs,[],[]),finfo) } 
>                   | SRules Commands                      {let 
>                                                               (rs,finfo) = $1 
>                                                               cs = [c | c <- $2, not $ isUNION c]
>                                                               ds = [c | c <- $2, isUNION c]
>                                                           in ((rs,cs,ds),finfo)}



> RulesAndCommands ::  {(([ConstrainedRule],[Command]),FunInfo)}
> RulesAndCommands : Rules                               { let (rs,finfo) = $1 in ((rs,[]),finfo) } 
>                  | Rules Commands                      { let 
>                                                               (rs,finfo) = $1 
>                                                               cs = $2
>                                                           in ((rs,cs),finfo)
>                                                         }
> Commands : Command                                       {[$1]}
>          | Command  Commands                             {$1:$2}
> Command  : 'overlap' RuleNameInCommand '.l' 'and' RuleNameInCommand '.l' {Single (Left $2) (Left $5) [] }
>          | 'overlap' RuleNameInCommand '.l' 'and' RuleNameInCommand '.r' {Single (Left $2) (Right $5)[]}
>          | 'overlap' RuleNameInCommand '.r' 'and' RuleNameInCommand '.l' {Single (Right $2) (Left $5)[]}
>          | 'overlap' RuleNameInCommand '.r' 'and' RuleNameInCommand '.r' {Single (Right $2) (Right $5)[]}
>          | 'overlap' RuleNameInCommand '.l' 'and' RuleNameInCommand '.l' 'ignore' VLIST {Single (Left $2) (Left $5) $8 }
>          | 'overlap' RuleNameInCommand '.l' 'and' RuleNameInCommand '.r' 'ignore' VLIST {Single (Left $2) (Right $5) $8}
>          | 'overlap' RuleNameInCommand '.r' 'and' RuleNameInCommand '.l' 'ignore' VLIST {Single (Right $2) (Left $5) $8}
>          | 'overlap' RuleNameInCommand '.r' 'and' RuleNameInCommand '.r' 'ignore' VLIST {Single (Right $2) (Right $5) $8}
>          | 'overlap' RuleNameInCommand '.r' 'all'          {All (Right $2) [] []}
>          | 'overlap' RuleNameInCommand '.l' 'all'          {All (Left $2) [] []}
>          | 'overlap' RuleNameInCommand '.r' 'all' 'except' VLIST {All (Right $2) $6 []}
>          | 'overlap' RuleNameInCommand '.l' 'all' 'except' VLIST {All (Left $2) $6 []}
>          | 'overlap' RuleNameInCommand '.r' 'all' 'ignore' VLIST {All (Right $2) [] $6}
>          | 'overlap' RuleNameInCommand '.l' 'all' 'ignore' VLIST {All (Left $2) [] $6}
>          | 'overlap' RuleNameInCommand '.r' 'all' 'except' VLIST 'ignore' VLIST {All (Right $2) $6 $8}
>          | 'overlap' RuleNameInCommand '.l' 'all' 'except' VLIST 'ignore' VLIST {All (Left $2) $6 $8}
>          | 'union' RuleNameInCommand '=' '{' VarList '}' {UNION $2 $5}

> VLIST : RuleNameInCommand               {[$1]}
>       | '{' VarList '}'   {$2}

>  VarList : RuleNameInCommand             {[$1]}
>         | RuleNameInCommand ',' VarList {$1:$3}

> SRules :: {(([ConstrainedRule],[ConstrainedRule],[ConstrainedRule]),FunInfo)}
> SRules : Rules                          { let (rs,finfo) = $1 
>                                           in (([r | r <- rs, standardReduction r, not (answerRule r)]
>                                               ,[r | r <- rs, not (standardReduction r), not (answerRule r)]
>                                               ,[r{ruleName=let (a,b,c,d)=(ruleName r) in if b == "ANSWER" then (a,b,n,d) else (a,b,c,d)} | (n,r) <- zip [1..] [r | r <- rs, answerRule r]]
>                                               ),finfo)}
>        | Rules 'TRANSFORMATIONS' Rules  {let 
>                                           (rs1,finfo1) = $1
>                                           (rs2,finfo2) = $3
>                                           rs1' = [r{standardReduction=True} | r <- rs1, not(answerRule r)]
>                                           rs2' = [r{standardReduction=False} | r <- rs2, not(answerRule r)]
>                                           rs3' = [r{standardReduction=False} | r <- (rs1 ++ rs2),(answerRule r)]
>                                           rs3'' = [r{ruleName=let (a,b,c,d)=(ruleName r) in if b == "ANSWER" then (a,b,n,d) else (a,b,c,d)} | (n,r) <- zip [1..] [r | r <- rs3']]
>                                          in ((rs1',rs2',rs3''),finfo1 ++ finfo2)
>                                         }


Rules = List of 4-Tupel (rule name, rule, list of nonempty contexts, list of nonempty enviroments)

> Rules :: {([ConstrainedRule],FunInfo)}
> Rules : RuleOrAnswer                         { let (r,finfo) = $1 in ([r],finfo)}
>       | RuleOrAnswer  Rules          { let 
>                                           (r,finfo1) = $1
>                                           (rs,finfo2) = $2
>                                        in (r:rs,finfo1 ++ finfo2)
>                                       }
> RuleNameInCommand :: {RuleName}
> RuleNameInCommand : '(' var                         ')'  {(False,$2,1,False)}
>                   | '(''SR' ',' var                 ')'  {(True,$4,1,False)}
>                   | '(' var ',' num                 ')'  {(False,$2,$4,False)}
>                   | '(''SR' ',' var ',' num         ')'  {(True,$4,$6,False)} 
>                   | '(' var ',' '+'                 ')'  {(False,$2,1,True)}
>                   | '(''SR' ',' var ',' '+' ',' num ')'  {(True,$4,$8,True)}
>                   | '(' var ',' '+' ',' num         ')'  {(False,$2,$6,True)}
>                   
> RuleName : '{' var '}'                               { (0,False,$2,1,False)  }
>          | '{' 'SR' ',' var '}'                      { (0,True,$4,1,False) }
>          | '{' num ',' 'SR' ',' var '}'              { ($2,True,$6,1,False) }
>          | '{' var ','  num '}'                      { (0,False,$2,$4,False)  }
>          | '{' 'SR' ',' var ',' num '}'              { (0,True,$4,$6,False) }
>          | '{' num ',' 'SR' ',' var ',' num '}'      { ($2,True,$6,$8,False) }
>          | '{' var ',' '+' '}'                       { (0,False,$2,1,True)  }
>          | '{' 'SR' ',' var ',' '+' '}'              { (0,True,$4,1,True) }
>          | '{' num ',' 'SR' ',' var ',' '+'  '}'     { ($2,True,$6,1,True) }
>          | '{' var ',' '+' ','  num '}'              { (0,False,$2,$6,True)  }
>          | '{' 'SR' ',' var ',' '+' ',' num '}'      { (0,True,$4,$8,True) }
>          | '{' num ',' 'SR' ',' var ',' '+' ',' num '}'      { ($2,True,$6,$10,True) }

> RuleOrAnswer : Rule                                           { $1 }
>              | 'ANSWER' '{' var ',' num '}' Answer            { let (r,f) = $7 in (r{ruleName=(False,$3,$5,False)},f) }
>              | 'ANSWER' '{' var '}' Answer                    { let (r,f) = $5 in (r{ruleName=(False,$3,0,False)},f)  }
>              | 'ANSWER' Answer                                { $2 }

> Answer :            Expr                              { let (e,finfo) = $1 
>                                                         in if isExpr e then
>                                                             (ConstrainedRule {
>                                                                ruleName = (False,"ANSWER",0,False)
>                                                               ,rule = e :==>: (Fn "ANSWER" [])
>                                                               ,nonEmptyCtxts = []
>                                                               ,nonEmptyEnvs  = []
>                                                               ,nonCapCon     = []
>                                                               ,priority = 0
>                                                               ,standardReduction = False
>                                                               ,answerRule = True
>                                                               }
>                                                              ,finfo 
>                                                              )
>                                                             else error ("expression " ++ show e ++ " is not an expression, but a context")
>                                                       }
>        | Expr 'where' Constraints                     { let (e,finfo) = $1 
>                                                         in if isExpr e then
>                                                             (ConstrainedRule {
>                                                                ruleName = (False,"ANSWER",0,False)
>                                                               ,rule = e :==>: (Fn "ANSWER" [])
>                                                               ,nonEmptyCtxts = ctxtcon $3
>                                                               ,nonEmptyEnvs  = envcon $3
>                                                               ,nonCapCon     = capcon $3  
>                                                               ,priority = 0
>                                                               ,standardReduction = False
>                                                               ,answerRule = True
>                                                               }
>                                                              ,finfo 
>                                                              )
>                                                             else error ("expression " ++ show e ++ " is not an expression, but a context")
>                                                       }
> Rule : RuleName Expr '==>' Expr  { let 
>                                           (e1,finfo1) = $2
>                                           (e2,finfo2) = $4
>                                        in
>                                         if isExpr e1 then 
>                                                       if isExpr e2 then 
>                                                                       (ConstrainedRule {
>                                                                                ruleName = let (prio,sr,name,number,trans) = $1 in (sr,name,number,trans) 
>                                                                               ,rule = e1 :==>: e2
>                                                                               ,nonEmptyCtxts = []
>                                                                               ,nonEmptyEnvs  = []
>                                                                               ,nonCapCon     = []
>                                                                               ,priority = let (prio,sr,name,number,trans) = $1 in prio
>                                                                               ,standardReduction = let (prio,sr,name,number,trans) = $1 in sr
>                                                                               ,answerRule = False
>                                                                               }
>                                                                       ,finfo1++finfo2
>                                                                       )
>                                                                    else error ("expression " ++ show e2 ++ " is not an expression, but a context")
>                                                       else error ("expression " ++ show e1 ++ " is not an expression, but a context")}
>
>      | RuleName  Expr '==>' Expr 'where' Constraints  { let 
>                                                               (e1,finfo1) = $2
>                                                               (e2,finfo2) = $4
>                                                            in if isExpr e1 then 
>                                                                if isExpr e2 then 
>                                                                       (ConstrainedRule {
>                                                                                ruleName = let (prio,sr,name,number,trans) = $1 in (sr,name,number,trans)
>                                                                               ,rule = e1 :==>: e2
>                                                                               ,nonEmptyCtxts = ctxtcon $6
>                                                                               ,nonEmptyEnvs  = envcon $6
>                                                                               ,nonCapCon     = capcon $6
>                                                                               ,priority = let (prio,sr,name,number,trans) = $1 in prio
>                                                                               ,standardReduction = let (prio,sr,name,number,trans) = $1 in sr
>                                                                               ,answerRule = False
>                                                                               }
>                                                                       ,finfo1++finfo2
>                                                                       )
>                                                                   else error ("expression " ++ show e2 ++ " is not an expression, but a context")
>                                                                 else error ("expression " ++ show e1 ++ " is not an expression, but a context")}


==========================================================================================
Parsing Equations with Constraints, to use for pure unification
==========================================================================================
==
==

EqConstraint = List of equations, list of nonempty contexts, list of nonempty environments

> EqConstr :: {(EquationsWithConstraints,FunInfo)}
> EqConstr : Equations                     { let (eqs,finfo)=$1 in ((eqs,[],[],[]),finfo)}
>          | Equations 'where' Constraints { let (eqs,finfo)=$1 in ((eqs,ctxtcon $3,envcon $3, capcon $3),finfo)}


> Equations :: {([Equation Expression],FunInfo)}
> Equations : Equation {let (e,finfo) = $1 in ([e],finfo)}
>           | Equation ',' Equations {let 
>                                       (e,finfo1) = $1 
>                                       (es,finfo2) = $3
>                                     in (e:es,finfo1++finfo2)}
> Equation : Expr '=?=' Expr    { let 
>                                   (e1,finfo1) = $1
>                                   (e2,finfo2) = $3
>                                 in
>                                   if isExpr e1 then 
>                                    if isExpr e2   then (e1 :=?=: e2,finfo1 ++ finfo2)
>                                                   else error ("expression " ++ show e2 ++ " is not an expression, but a context" )
>                                    else error ("expression " ++ show e1 ++ " is not an expression, but a context" )
>                               }

==
==
==========================================================================================


==========================================================================================
Parsing Equations with Constraints, to use for matching (experimental)
==========================================================================================
==
==


> EqTwoConstr :: {(EquationsWithTwoConstraints,FunInfo)}
> EqTwoConstr : Equations                     { let (eqs,finfo)=$1 in ((eqs,[],[],[],[],[],[]),finfo)}
>          | Equations 'where' '|' Constraints { let (eqs,finfo)=$1 in ((eqs,[],[],[],ctxtcon $4,envcon $4,capcon $4),finfo) }
>          | Equations 'where' Constraints '|' { let (eqs,finfo)=$1 in ((eqs,ctxtcon $3,envcon $3, capcon $3,[],[],[]),finfo)  }
>          | Equations 'where' Constraints '|' Constraints { let (eqs,finfo)=$1 in ((eqs,ctxtcon $3,envcon $3, capcon $3,ctxtcon $5,envcon $5,capcon $5),finfo)}



==
==
==========================================================================================


==========================================================================================
Parsing Expressions
together with strictness information of function symbol positions
==========================================================================================
==
==



== Productions for Expr

> Expr :: {(Expression,FunInfo)}
> Expr : '\\' LamVar '.' Expr           { let 
>                                           lamv = $2
>                                           (e,finfo) = $4 
>                                         in (Fn "\\" [Binder [lamv] e],("\\",1,[],[1]):finfo) 
>                                       }
>
>      | 'letrec' '{' Env '}' 'in' Expr { let 
>                                           (e,finfo) = $6
>                                           (es,finfo') = $3 
>                                         in (Letrec (applyEnvItems es empty) e, finfo ++ finfo') 
>                                       }
> 
>      | 'letrec' Env 'in' Expr         { let 
>                                           (e,finfo) = $4 
>                                           (es,finfo') = $2
>                                         in (Letrec (applyEnvItems es empty) e, finfo ++ finfo')
>                                       }
>      | SExpr                          { $1 }                   

== Productions for SExpr

> SExpr :: {(Expression,FunInfo)}
> SExpr : var LExpr                     {let 
>                                           (argsWithBindArities,finfo) = $2 
>                                           rev = reverse argsWithBindArities
>                                           args = map fst rev
>                                           bindarities = map snd rev
>                                        in   (Fn $1 args,($1, length args,[1],bindarities):finfo) -- default strictness!
>                                       }
>       | '\\' '(' LamVar '.' Expr ')'  { let 
>                                           lamv = $3
>                                           (e,finfo) = $5 
>                                         in (Fn "\\" [Binder [lamv] e],("\\",1,[],[1]):finfo) 
>                                       }
>       | var                           {(Fn $1 [],[($1,0,[],[])])}
>       | var '[' ']'                   {(Fn $1 [],[($1,0,[],[])])}
>       | var '[' SPos ']' LExpr        { let 
>                                            (argsWithBindArities,finfo) = $5 
>                                            rev = reverse argsWithBindArities
>                                            args = map fst rev
>                                            bindarities = map snd rev
>                         
>                                         in (Fn $1 args,($1,length args,$3,bindarities):finfo)
>                                       }
>       | var '['  ']' LExpr            { let 
>                                            (argsWithBindArities,finfo) = $4 
>                                            rev = reverse argsWithBindArities
>                                            args = map fst rev
>                                            bindarities = map snd rev
>                                         in (Fn $1 args,($1,length args,[],bindarities):finfo)
>                                       }
>       | Atom                          { $1 }
 



> LExpr :: {([(ArgPos,BindArity)],FunInfo)}
> LExpr : Binder                           { let (a,finfo,barity) = $1 in ([(a,barity)],finfo)  }
>       | LExpr Binder                     { let
>                                               (as,finfo) = $1
>                                               (a,finfo',barity) = $2
>                                           in ((a,barity):as,finfo++finfo')
>                                          }

== Productions for Binder

> Binder :: {(ArgPos,FunInfo,BindArity)}
> Binder : Atom                            { let (e,finfo) = $1 in  (Binder [] e,finfo,0) }
>        | LamVar '.' Binder               { let (Binder xs e,finfo,i) = $3 in (Binder ($1:xs) e,finfo,i+1) }
>        | '*' LamVar                      { (VarPos $2,[],-1)}
>        | '(' '*' LamVar ')'              { (VarPos $3,[],-1)}


== Productions for Atom

> Atom :: {(Expression,FunInfo)}
> Atom:  'vlam' LamVar                     { (VarL $2, [])}
>      | 'vlet' LamVar                     { (VarL $2, [])}
>      | 'vlift' LamVar                    { (VarL $2, [])}
>      | 'var' LamVar                    { (VarL $2, [])}
>      | metavar                           { (MetaExpression emptySubstitution $1,[])}
>      | metavarACtxt                      { (MetaCtxt emptySubstitution ACtxt $1,[])}
>      | metavarACtxt '[' Expr ']'         { let
>                                               (e,finfo) = $3
>                                            in (MetaCtxtSubst emptySubstitution ACtxt $1 e,finfo)
>                                          }
>      | metavarTCtxt                      { (MetaCtxt emptySubstitution TCtxt $1,[]) }
>      | metavarTCtxt '[' Expr ']'         { let 
>                                               (e,finfo) = $3 
>                                            in (MetaCtxtSubst emptySubstitution TCtxt $1 e,finfo)
>                                          }
>      | metavarCCtxt                      { (MetaCtxt emptySubstitution Ctxt $1,[])}
>      | metavarCCtxt '[' Expr ']'         { let
>                                              (e,finfo) = $3
>                                            in (MetaCtxtSubst emptySubstitution Ctxt $1 e,finfo)
>                                          }
>      | '(' Expr ')'                      { $2 }
>      | 'HOLE'                            { (Hole emptySubstitution,[]) }


== Productions for LamVar 

> LamVar  : metavar                         { MetaVarLift emptySubstitution $1 } 
>         | var                             { VarLift emptySubstitution  $1 }

== Productions for LetVar 


> Env : EnvItem                                 { let (e,finfo) = $1 in  ([e],finfo) }
>     | EnvItem ';' Env                         { let 
>                                                   (e,finfo) = $1
>                                                   (es,finfo') = $3
>                                                 in (e:es,finfo ++ finfo') 
>                                               }

> EnvItem : metavar                             { (A $1, []) }
>         | metavar '[' LamVar ',' Expr ']'     { let (e,finfo) = $5 in (B (MetaChain emptySubstitution  AChain $1 $3 e),finfo) }
>         | metavar '|' LamVar ',' LamVar '|'   { (B (MetaChain emptySubstitution  VarChain $1 $3 (VarL $5)),[]) }
>         | LamVar '=' Expr                     { let (e,finfo) = $3 in (C ($1 := e),finfo) }

== Productions for SPos

> SPos :: {[Int]}
> SPos : num            {[$1]}
>      | num ',' SPos   {$1:$3}


==
==
==========================================================================================


==========================================================================================
Parsing Constraints
==========================================================================================
==
==

> Constraints : Constraint {[$1]}
>             | Constraint ',' Constraints {$1:$3}
> Constraint :  metavar  '/=' '{' '}'   {EnvConstraint  $1}
>            |  metavarACtxt '/=' '['']'    {CtxtConstraint $1}
>            |  metavarCCtxt '/=' '['']'    {CtxtConstraint $1}
>            |  metavarTCtxt '/=' '['']'    {CtxtConstraint $1}
>            |  '(' Expr ',' Expr ')'   {let 
>                                            (e1,_) = $2 
>                                            (e2,_) = $4 
>                                        in CapConstraint  (e1,e2) }
>            |  '[' Env ',' Expr ']'    {let 
>                                            (e1,_) = $2 
>                                            (e2,_) = $4 
>                                        in CapConstraintEnv  (e1,e2) }

==
==
==========================================================================================


> {

help functions, for computing the enviroment and context constraints

> data Constraint a b c d = EnvConstraint a | CtxtConstraint b | CapConstraint c | CapConstraintEnv d
> 
> envcon xs = [Env emptySubstitution x | (EnvConstraint x) <- xs]
> ctxtcon xs = [MetaCtxt emptySubstitution (srt x) x | (CtxtConstraint x) <- xs]
> capcon xs = [(Left a,b) | (CapConstraint (a,b)) <- xs]
>            ++ [(Right $ applyEnvItems a empty,b) | CapConstraintEnv (a,b) <- xs]
>

help functions, for computing the parts of an enviroment

> data EnvItem a b c = A a | B b | C c
> applyEnvItems xs env = 
>   env{metaEnvVars = map (Env emptySubstitution) (map fromA $ filter isA xs),
>       metaChains  = map fromB $ filter isB xs,
>       bindings    = map fromC $ filter isC xs}
>  where
>   isA (A _) = True
>   isA _ = False
>   isB (B _) = True
>   isB _ = False
>   isC (C _) = True
>   isC _ = False
>   fromA (A x) = x
>   fromB (B x) = x
>   fromC (C x) = x
> srt ('A':_) = ACtxt
> srt ('T':_) = TCtxt
> srt ('C':_) = Ctxt
> srt _ = Ctxt

the error function

> happyError :: [TokenType] -> a
> happyError []     = error $ "parse error: unexpected end of file "
> happyError (x:xs) = error $ "parse error before: "
>                           ++ (take 200 $ concat ( intersperse " " ( map showToken xs)))  ++ (printMark xs)



> parser str =
>  let (parseRes,funinfo) = parse $ lexInput str
>  in  case (check (parseResToProblem parseRes)) of 
>        Just err -> error err
>        Nothing -> (parseRes,funinfo)

> parseResToProblem (eqs,ctxtcon,envcon,ncc) =
>    Problem {                         solution       = []
>                                     ,gamma          = gam
>                                     ,delta          = Delta {delta1=ctxtcon,delta2=envcon,delta3=ncc,delta4=emptyGamma}
>                                     ,fresh          = []
>                                     ,functions      = []
>                                     ,inputGamma     = gam
>                                     ,inputDelta     = Delta {delta1=ctxtcon,delta2=envcon,delta3=ncc,delta4=emptyGamma}
>                                     ,constantDelta  = emptyDelta
>                                 }
>   where gam =  Gamma { varLiftEq = []
>                      , expressionEq = eqs
>                      , environmentEq = []
>                      , bindingEq = []
>                      } 

> parserMatching str =
>  let (parseRes,funinfo) = parseMatchProblem $ lexInput str
>  in  (parseRes,funinfo)



> compactFunInfo :: FunInfo -> FunInfo
> compactFunInfo finfo =
>  let 
>       sorted = sortBy (\(n,arity,spos,bari) (n1,arity1,spos1,bari1) -> compare n n1) finfo
>       grouped = groupBy (\(n,arity,spos,bari) (n1,arity1,spos1,bari1) -> n == n1) sorted
>       joinGroup group@((n,arity,spos,bari):_) = 
>                    let 
>                       arities = [a | (_,a,_,_) <- group]
>                       spositions = [s | (_,_,s,_) <- group]
>                       lspos      = [l | l <- spositions, l /= [1]]
>                       bindarities  = [b | (_,_,_,b) <- group]
>                       lbindarities = [b | b <- bindarities, b /= []]
>                       theArity = if (length (nub arities) /= 1) then
>                                       error $ "function symbol '" ++ n ++ "' occurs with different arities"
>                                   else arity
>                       theSpos  = if null lspos then [1] else
>                                   if (length (nub lspos)) == 1 then (head lspos) else 
>                                       error $ "function symbol '" ++ n ++ "' occurs with different strict positions"
>                       theBindArities = if null lbindarities then [] else
>                                         if (length (nub lbindarities)) == 1 then (head lbindarities) else
>                                       error $ "function symbol '" ++ n ++ "' occurs with different binder arities"
>                          
>                    in (n,theArity,theSpos,theBindArities)
>  in map joinGroup grouped
> isUNION (UNION _ _) = True
> isUNION _ = False



> }


